import numpy as np


def window1d(
    input_array: list | np.ndarray, size: int, shift: int = 1, stride: int = 1
) -> list[list | np.ndarray]:
    """
    This function performs a windowing operation on a 1-dimensional input array.

    Parameters:
    input_array (Union[List[float], np.ndarray]): The 1-dimensional input array consisting of numbers.
    size (int): The size of each window. This must be a positive integer.
    shift (int, optional): The number of positions to shift the window at each step. This must be a positive integer. Defaults to 1.
    stride (int, optional): The number of positions to step within each window. This must be a positive integer. Defaults to 1.

    Returns:
    List[Union[List[float], np.ndarray]]: A list of numpy arrays, where each array is a window in the original array.

    Raises:
    TypeError: If input_array is not a list or numpy array.
    ValueError: If input_array does not contain only integers or floats, or if size, shift, or stride are not positive integers.

    Example:
    >>> window1d([1, 2, 3, 4, 5, 6, 7, 8, 9, 10], 5, 2, 2)
    [array([1., 3., 5.]), array([3., 5., 7.]), array([5., 7., 9.])]
    """
    # Input validation
    if not isinstance(input_array, (list, np.ndarray)):
        raise TypeError("input_array must be a list or numpy array")
    if not all(isinstance(i, (int, float, np.number)) for i in input_array):
        raise ValueError("input_array must contain only numbers")
    if not isinstance(size, int) or size < 1:
        raise ValueError("size must be an integer greater than 0")
    if not isinstance(shift, int) or shift < 1:
        raise ValueError("shift must be an integer greater than 0")
    if not isinstance(stride, int) or stride < 1:
        raise ValueError("stride must be an integer greater than 0")

    # Convert input_array to numpy array if it's not
    input_array = np.array(input_array, dtype=float)

    # Initialize the output list
    output = []

    # Iterate over the input_array with a step size of shift
    for i in range(0, len(input_array) - size * stride + 1, shift):
        window = input_array[i : i + size : stride]
        output.append(window)

    # Return the output list
    return output


# Test the function
if __name__ == "__main__":
    try:
        length = int(input("Enter the length of the array: "))
        size = int(input("Enter the window size: "))
        shift = int(input("Enter the window shift size: "))
        stride = int(input("Enter the window stride size: "))

        array = []
        for i in range(length):
            array.append(float(input(f"Enter element at position {i + 1}: ")))

        print("Original Array:")
        print(array)

        print("\nWindowed Array:")
        windowed_array = window1d(array, size, shift, stride)
        for window in windowed_array:
            print(window)
    except Exception as e:
        print(f"An error occurred: {e}")
